// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _MUTEX_H_
#define _MUTEX_H_

#include <pthread.h>
#include <errno.h>
#include <string>
#include <map>

#include <silecs-communication/interface/utility/SilecsException.h>

namespace Silecs
{

class Condition;

class Mutex
{
public:
    Mutex(std::string name = "undefined");
    ~Mutex();

    void lock();
    void unlock();

private:
    friend class Condition;

    // Mutex name if any, used for logging in particular
    std::string name_;

    // Mutex object
    pthread_mutex_t* pMutex_;

    // Mutex Attributes
    pthread_mutexattr_t attr_;
};

// For automatic mutex-release on code-block exit ( e.g. if exceptions are used )
class Lock
{
public:

    Lock(Mutex* mutex) :
                    mutex_(mutex)
    {
        mutex_->lock();
    }

    Lock() :
                    mutex_(NULL)
    {
    }

    ~Lock()
    {
        if (mutex_ != NULL)
            mutex_->unlock();
    }

    void setMutex(Mutex* mutex)
    {
        mutex_ = mutex;
    }

protected:

    Mutex* mutex_;
};

} // namespace

#endif /* _MUTEX_H_ */
