// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SILECS_DIAG_SILECSMODULE_H
#define SILECS_DIAG_SILECSMODULE_H

#include <QtGui>
#include <QtCore>

#include <silecs-communication/interface/core/SilecsService.h>

#include <silecs-diagnostic/constants.h>
#include <silecs-diagnostic/diagnostictoolmainview.h>
#include <silecs-diagnostic/item.h>
#include <silecs-diagnostic/utils.h>

#include <string>
#include <cstdlib>
#include <iostream>
#include <sstream>
#include <algorithm>

using namespace Silecs;

enum LogTopic{ERROR, DEBUG, SETUP, ALLOC, LOCK, COMM, SEND, RECV, DATA, DIAG };

class silecsModule
{
private:
    Silecs::Service* silecsService;
    Silecs::Cluster* silecsCluster;

    /**
      * Add an element to the tree
      */
    QTreeWidgetItem* addTreeItem(QTreeWidgetItem *parent,QString name ,QString description);

public:

    /**
      * Constructor. Get the SILECS service from the library
      */
    silecsModule(std::string logParam, QTextBrowser *messageConsole);

    /**
      * setLogTopics from the service
      */
    void setLogTopics(std::string logParam);

    bool isDebugLogEnabled();
    void enableDebugLog();

    /**
      * Denstructor. Remove the SILECS service from the library
      */
    ~silecsModule();

    void updatePLCRunState(Item *plcItem);

    /**
      *Generate the empty entire tree
      */
    Item *generateTree(string className, string deployFile);

    /**
      * Set the device with the proper scalar data converting
      * from the string present in the GUI
      * in case of error returns a string exception with the message
      */
    void setScalarDataInDeviceFromItem(Item *currentItem, std::string blockName) throw (std::string*);

    /**
      * Set the register with the proper data converting
      * from the string present in the GUI
      * in case of error returns a string exception with the message
      */
    void setArrayRegValue(Item *currentItem,std::vector<QString> *dataVector);

    /**
     * Updates the input and output values of all the register
     * for the given device Item without any other impact on the GUI
     */
    void updateDeviceItem(Item *deviceItem,bool updateInputBufferOnly);

    /**
     * Updates the input and output values of all the register
     * for the given PLC Item without any other impact on the GUI
     */
    void updatePLCItem(Item *PLCItem,bool updateInputBufferOnly);

    /**
     * Updates the input and output values of all the register
     * for the given cluster Item without any other impact on the GUI
     */
    void updateClusterItem(Item *Cluster,bool updateInputBufferOnly);

    /**
      * Count the number of PLC connected
      */
    int counterConnectedPLC;

    QTextBrowser *messageConsole_;

private:

    bool debugLoggingEnabled_;

};

#endif // SILECS_DIAG_SILECSMODULE_H
